#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/commands/deploy.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from re import match
from time import sleep
from typing import Dict, List

### Third-party packages ###
from click import command, option
from docker import DockerClient, from_env
from docker.errors import APIError, DockerException, NotFound
from docker.models.containers import Container
from pydantic import TypeAdapter
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from src.configs import CLUSTERS, IMAGES, NETWORK, PERIPHERALS
from src.schemas import MutexOption, NewAddress, Service, ServiceName


@command
@option("--duo", alternatives=["uno"], cls=MutexOption, is_flag=True, type=bool)
@option("--uno", alternatives=["duo"], cls=MutexOption, is_flag=True, type=bool)
@option("--with-postgres", is_flag=True, help="Deploy postgres peripheral service", type=bool)
@option("--with-redis", is_flag=True, help="Deploy redis peripheral service", type=bool)
def deploy(duo: bool, uno: bool, with_postgres: bool, with_redis: bool) -> None:
    """Deploy cluster, either with one or two LND nodes."""
    client: DockerClient
    try:
        client = from_env()
        if not client.ping():
            raise DockerException
    except DockerException:
        rich_print("[red bold]Unable to connect to docker daemon.")
        return

    ### Defaults to duo network; Derive cluster information from parameters ###
    duo = duo or (not duo and not uno)  # defaults to duo network
    cluster: Dict[ServiceName, Service] = (CLUSTERS["duo"], CLUSTERS["uno"])[uno]
    peripheral_select: Dict[str, bool] = {"postgres": with_postgres, "redis": with_redis}
    peripherals: Dict[ServiceName, Service] = {
        f"aesir-{k}": v[f"aesir-{k}"] for k, v in PERIPHERALS.items() if peripheral_select[k]  # type: ignore[index, misc]
    }
    cluster.update(peripherals)

    ### Attempts to create network if not exist ###
    try:
        client.networks.create(NETWORK, check_duplicate=True)
    except APIError:
        pass

    ### Deploy specified cluster ###
    for name, service in track(cluster.items(), f"Deploy {('duo', 'uno')[uno]} cluster:".ljust(42)):
        image_name: str = dict(**IMAGES["required"], **IMAGES["optional"])[service.alias]
        ports: Dict[str, str] = dict(
            map(lambda item: (item[0], item[1]), [port.split(":") for port in service.ports])
        )
        client.containers.run(
            image_name,
            command=service.command,
            detach=True,
            environment=service.env_vars,
            name=name,
            network=NETWORK,
            ports=ports,
        )

    ### Wait until lnd(s) ready ###
    sleep(3)

    ### Mine starting capital ###
    treasuries: List[str] = []
    for container in track(client.containers.list(), "Generate addresses:".ljust(42)):
        if match(r"aesir-lnd|aesir-ping|aesir-pong", container.name) is not None:
            new_address: NewAddress = TypeAdapter(NewAddress).validate_json(
                container.exec_run(
                    """
                    lncli
                        --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                        --rpcserver=localhost:10001
                        --tlscertpath=/home/lnd/.lnd/tls.cert
                    newaddress p2wkh
                    """
                ).output
            )
            treasuries.append(new_address.address)

    ### Retrieve bitcoind container ###
    bitcoind: Container
    try:
        bitcoind = client.containers.get("aesir-bitcoind")
    except NotFound:
        rich_print('[dim yellow1]Unable to find "aesir-bitcoind"; initial capital not yet mined.')
        return
    for address in track(treasuries, "Mine initial capital for parties:".ljust(42)):
        bitcoind.exec_run(
            """
            bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir generatetoaddress 101 %s
            """
            % address
        )


__all__ = ["deploy"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/__init__.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Local modules ###
from src.commands import clean, deploy, flush, mine, nodekeys, ping_pong, setup
from src.schemas import (
    BlockchainInfo,
    ClusterEnum,
    ImageAlias,
    ImageEnum,
    LNDInfo,
    MutexOption,
    NewAddress,
    OpenChannel,
    Service,
    ServiceName,
)


__all__ = [
    "ClusterEnum",
    "BlockchainInfo",
    "LNDInfo",
    "ImageAlias",
    "ImageEnum",
    "MutexOption",
    "NewAddress",
    "OpenChannel",
    "Service",
    "ServiceName",
    "clean",
    "deploy",
    "flush",
    "mine",
    "nodekeys",
    "ping_pong",
    "setup",
]

__version__ = "0.2.9"



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/commands/setup.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 06:18
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import Dict, List, Set

### Third-party packages ###
from click import command, option
from docker import DockerClient, from_env
from docker.errors import DockerException
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from src.configs import IMAGES


@command
@option("--with-postgres", is_flag=True, help="Pull postgres optional image", type=bool)
@option("--with-redis", is_flag=True, help="Pull redis optional image", type=bool)
def setup(with_postgres: bool, with_redis: bool) -> None:
    """Download docker images used by command-line interface."""
    client: DockerClient
    try:
        client = from_env()
        if not client.ping():
            raise DockerException
    except DockerException:
        rich_print("[red bold]Unable to connect to docker daemon.")
        return

    ### Pull required images ###
    outputs: List[str] = []
    docker_images: Set[str] = {image.tags[0] for image in client.images.list()}
    for registry_id in track(IMAGES["required"].values(), "Pull required images:".ljust(42)):
        if registry_id in docker_images:
            outputs.append(f"<Image: '{ registry_id }'> already exists in local docker images.")
        else:
            repository, tag = registry_id.split(":")
            client.images.pull(repository=repository, tag=tag)
            outputs.append(f"<Image: '{ registry_id }'> downloaded.")
    list(map(rich_print, outputs))

    ### Pull optional images ###
    optional_select: Dict[str, bool] = {"postgres": with_postgres, "redis": with_redis}
    optionals: List[str] = [
        registry for alias, registry in IMAGES["optional"].items() if optional_select[alias]
    ]
    if len(optionals) == 0:
        return
    outputs = []
    for registry_id in track(optionals, "Pull optional images flagged:".ljust(42)):
        if registry_id in docker_images:
            outputs.append(f"<Image: '{ registry_id }'> already exists in local docker images.")
        else:
            repository, tag = registry_id.split(":")
            client.images.pull(repository=repository, tag=tag)
            outputs.append(f"<Image: '{ registry_id }'> downloaded.")
    list(map(rich_print, outputs))


__all__ = ["setup"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/schemas/__init__.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Local modules ###
from src.schemas.blockchain_info import BlockchainInfo
from src.schemas.cluster_enum import ClusterEnum
from src.schemas.image import ImageAlias, ImageEnum
from src.schemas.lnd_info import LNDInfo
from src.schemas.mutex_option import MutexOption
from src.schemas.new_address import NewAddress
from src.schemas.open_channel import OpenChannel
from src.schemas.peripheral_enum import PeripheralEnum
from src.schemas.service import Service, ServiceName


__all__ = [
    "BlockchainInfo",
    "ClusterEnum",
    "ImageAlias",
    "ImageEnum",
    "LNDInfo",
    "MutexOption",
    "NewAddress",
    "OpenChannel",
    "PeripheralEnum",
    "Service",
    "ServiceName",
]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/configs.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from pathlib import Path
from typing import Any, Dict, List, Optional

### Standard packages ###
from pydantic import TypeAdapter
from yaml import Loader, load

### Local modules ###
from src.schemas import ClusterEnum, ImageAlias, ImageEnum, PeripheralEnum, Service, ServiceName

CLUSTERS: Dict[ClusterEnum, Dict[ServiceName, Service]]
DEPRECATED: List[str]
IMAGES: Dict[ImageEnum, Dict[ImageAlias, str]]
NETWORK: str
PERIPHERALS: Dict[PeripheralEnum, Dict[ServiceName, Service]]

file_path: Path = Path(__file__).resolve()
with open(str(file_path).replace("configs.py", "constants.yaml"), "rb") as stream:
    constants: Optional[Dict[str, Any]] = load(stream, Loader=Loader)
    if constants:
        CLUSTERS = TypeAdapter(Dict[ClusterEnum, Dict[ServiceName, Service]]).validate_python(
            constants["clusters"]
        )
        DEPRECATED = constants.get("deprecated", [])
        IMAGES = TypeAdapter(Dict[ImageEnum, Dict[ImageAlias, str]]).validate_python(
            constants["images"]
        )
        NETWORK = constants.get("network", "aesir")
        PERIPHERALS = TypeAdapter(Dict[PeripheralEnum, Dict[ServiceName, Service]]).validate_python(
            constants["peripherals"]
        )

__all__ = ["CLUSTERS", "DEPRECATED", "IMAGES", "NETWORK", "PERIPHERALS"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:     ~~/src/commands/mine.py
# VERSION: 	    0.2.9
# CREATED: 	    2023-12-01 05:31
# AUTHOR:       Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from re import match
from typing import List

### Third-party packages ###
from apscheduler.schedulers.background import BackgroundScheduler
from click import argument, command
from docker import DockerClient, from_env
from docker.errors import DockerException, NotFound
from docker.models.containers import Container
from pydantic import TypeAdapter
from rich import print as rich_print
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.progress import track
from rich.table import Table
from rich.text import Text

### Local modules ###
from src.schemas import BlockchainInfo, LNDInfo, NewAddress


@command
@argument("blockcount", default=1, type=int)
@argument("blocktime", default=5, type=int)
def mine(blockcount: int, blocktime: int) -> None:
    """Scheduled mining with "blockcount" and "blocktime"."""
    client: DockerClient
    try:
        client = from_env()
        if not client.ping():
            raise DockerException
    except DockerException:
        rich_print("[red bold]Unable to connect to daemon.")
        return

    ### Retrieve bitcoind container ###
    bitcoind: Container
    try:
        bitcoind = client.containers.get("aesir-bitcoind")
    except NotFound:
        rich_print('[red bold]Unable to find "aesir-bitcoind" container.')
        return

    ### Retrieve other containers ###
    aesir_containers: List[str] = list(
        filter(
            lambda container: match(r"aesir-*", container.name), reversed(client.containers.list())
        )
    )
    container_names: List[str] = map(lambda container: container.name, aesir_containers)
    lnd_containers: List[Container] = list(
        filter(
            lambda container: match(r"aesir-lnd|aesir-ping|aesir-pong", container.name),
            aesir_containers,
        )
    )

    ### Generate treasury addresses as mining destinations ###
    treasuries: List[str] = []
    for container in track(lnd_containers, "Generate mining treasuries:".ljust(42)):
        new_address: NewAddress = TypeAdapter(NewAddress).validate_json(
            container.exec_run(
                """
                lncli
                    --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                    --rpcserver=localhost:10001
                    --tlscertpath=/home/lnd/.lnd/tls.cert
                newaddress p2wkh
                """
            ).output
        )
        treasuries.append(new_address.address)

    ### Set up mining schedule using command arguments ###
    scheduler: BackgroundScheduler = BackgroundScheduler()
    for address in treasuries:
        scheduler.add_job(
            bitcoind.exec_run,
            "interval",
            [
                """
                bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir generatetoaddress %d %s
                """
                % (blockcount, address)
            ],
            seconds=blocktime,
        )
    scheduler.start()

    ### Set up dashboard layout ###
    pane: Layout = Layout()
    sidebar: Layout = Layout(size=24)
    containers: Layout = Layout(name="containers", size=20)
    main: Layout = Layout(size=72)
    body: Layout = Layout(name="body", minimum_size=4, ratio=8, size=17)
    footer: Layout = Layout(name="footer", size=3)
    pane.split_row(sidebar, main)
    main.split_column(body, footer)
    sidebar.split_column(containers)

    with Live(pane, refresh_per_second=4, transient=True):
        pane["containers"].update(Panel(Text("\n".join(container_names)), title="containers"))
        while True:
            ### Update ###
            blockchain_info: BlockchainInfo = TypeAdapter(BlockchainInfo).validate_json(
                bitcoind.exec_run(
                    """
                    bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir getblockchaininfo
                    """
                ).output
            )

            ### Initiate parameters ###
            lnd_infos: List[LNDInfo] = []
            lnd_names: List[str] = []

            ## Fetch LNDInfo from LND containers ###
            for container in lnd_containers:
                lnd_info: LNDInfo = TypeAdapter(LNDInfo).validate_json(
                    container.exec_run(
                        """
                        lncli
                            --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                            --rpcserver=localhost:10001
                            --tlscertpath=/home/lnd/.lnd/tls.cert
                        getinfo
                        """
                    ).output
                )
                lnd_infos.append(lnd_info)
                lnd_names.append(container.name)

            ### Draw ###
            body_table: Table = Table(expand=True, show_lines=True)
            body_table.add_column("Name", justify="left", style="green")
            body_table.add_column("Nodekey", justify="left", style="cyan")
            body_table.add_column("Channels")
            body_table.add_column("Peers")
            body_table.add_column("Height")
            body_table.add_column("Synced?", justify="right")
            for i, lnd_info in enumerate(lnd_infos):
                body_table.add_row(
                    lnd_names[i],
                    "\n".join(
                        lnd_info.identity_pubkey[c : c + 11]
                        for c in range(0, len(lnd_info.identity_pubkey), 11)
                    ),
                    f"{lnd_info.num_active_channels}",
                    f"{lnd_info.num_peers}",
                    f"{lnd_info.block_height}",
                    ("[red]false", "[green]true")[lnd_info.synced_to_chain],
                )
            pane["body"].update(body_table)
            pane["footer"].update(
                Panel(
                    Text.assemble(
                        ("Chain: ", "bright_magenta bold"),
                        blockchain_info.chain.ljust(9),
                        ("Blocks: ", "green bold"),
                        f"{blockchain_info.blocks}".ljust(8),
                        ("Size: ", "blue bold"),
                        f"{blockchain_info.size_on_disk}".ljust(10),
                        ("Time: ", "cyan bold"),
                        f"{blockchain_info.time}".rjust(10),
                    )
                )
            )


__all__ = ["mine"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/schemas/mutex_option.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import Any, List, Mapping, Tuple

### Third-party packages ###
from click import Context, Option, UsageError


class MutexOption(Option):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        self.alternatives: list = kwargs.pop("alternatives")
        assert self.alternatives, "'alternatives' parameter required."
        kwargs["help"] = (
            kwargs.get("help", "")
            + f"Option is mutually exclusive with {', '.join(self.alternatives)}."
        ).strip()
        super(MutexOption, self).__init__(*args, **kwargs)

    def handle_parse_result(
        self, context: Context, options: Mapping[str, Any], arguments: List[str]
    ) -> Tuple[Any, List[str]]:
        current_opt: bool = self.name in options
        for mutex_option in self.alternatives:
            if mutex_option in options:
                if current_opt:
                    raise UsageError(
                        f"Illegal usage: '{self.name}' is mutually exclusive with {mutex_option}."
                    )
                else:
                    self.prompt = None
        return super(MutexOption, self).handle_parse_result(context, options, arguments)


__all__ = ["MutexOption"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/commands/nodekeys.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from re import match
from typing import List

### Third-party packages ###
from click import command
from docker import DockerClient, from_env
from docker.errors import DockerException
from docker.models.containers import Container
from pydantic import TypeAdapter
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from src.schemas import LNDInfo


@command
def nodekeys() -> None:
    """Fetch nodekeys from active LND containers."""
    client: DockerClient
    try:
        client = from_env()
        if not client.ping():
            raise DockerException
    except DockerException:
        rich_print("[red bold]Unable to connect to docker daemon.")
        return

    lnds: List[Container] = list(
        filter(
            lambda container: match(r"aesir-lnd|aesir-ping|aesir-pong", container.name),
            reversed(client.containers.list()),
        )
    )
    outputs: List[str] = []
    for container in track(lnds, "Fetch LND nodekeys:".ljust(42)):
        lnd_info: LNDInfo = TypeAdapter(LNDInfo).validate_json(
            container.exec_run(
                """
                lncli
                    --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                    --rpcserver=localhost:10001
                    --tlscertpath=/home/lnd/.lnd/tls.cert
                getinfo
                """
            ).output
        )
        outputs.append(f"<Nodekey: '{ container.name }', '{ lnd_info.identity_pubkey }'>")
    list(map(rich_print, outputs))


__all__ = ["nodekeys"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/schemas/blockchain_info.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Third-party packages ###
from pydantic import BaseModel, StrictInt, StrictStr


class BlockchainInfo(BaseModel):
    blocks: StrictInt
    chain: StrictStr
    size_on_disk: StrictInt
    time: StrictInt


__all__ = ["BlockchainInfo"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/commands/__init__.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Local modules ###
from src.commands.clean import clean
from src.commands.deploy import deploy
from src.commands.flush import flush
from src.commands.mine import mine
from src.commands.nodekeys import nodekeys
from src.commands.ping_pong import ping_pong
from src.commands.setup import setup

__all__ = ["clean", "deploy", "flush", "mine", "nodekeys", "ping_pong", "setup"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/commands/ping_pong.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 06:18
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from re import match
from typing import Dict, List

### Third-party packages ###
from click import argument, command
from docker import DockerClient, from_env
from docker.errors import DockerException, NotFound
from docker.models.containers import Container
from pydantic import TypeAdapter, ValidationError
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from src.schemas import LNDInfo, NewAddress, OpenChannel


@command
@argument("channel_size", default=16777215)
def ping_pong(channel_size: int) -> None:
    """For "duo" cluster, create channels between LND nodes."""
    client: DockerClient
    try:
        client = from_env()
        if not client.ping():
            raise DockerException
    except DockerException:
        rich_print("[red bold]Unable to connect to docker daemon.")
        return

    ### Fetch bitcoind container ###
    bitcoind: Container
    try:
        bitcoind = client.containers.get("aesir-bitcoind")
    except NotFound:
        rich_print('[red bold]Unable to find "aesir-bitcoind" container.')
        return

    ### Initiate parameters ###
    nodekeys: Dict[str, str] = {}
    paddles: List[Container] = list(
        filter(
            lambda container: match(r"aesir-ping|aesir-pong", container.name),
            reversed(client.containers.list()),
        )
    )
    treasuries: Dict[str, str] = {}

    ### Fetch nodekeys ###
    for container in track(paddles, "Fetch LND nodekeys:".ljust(42)):
        lnd_info: LNDInfo = TypeAdapter(LNDInfo).validate_json(
            container.exec_run(
                """
                lncli
                    --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                    --rpcserver=localhost:10001
                    --tlscertpath=/home/lnd/.lnd/tls.cert
                getinfo
                """
            ).output
        )
        nodekeys[container.name] = lnd_info.identity_pubkey
        new_address: NewAddress = TypeAdapter(NewAddress).validate_json(
            container.exec_run(
                """
                lncli
                    --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                    --rpcserver=localhost:10001
                    --tlscertpath=/home/lnd/.lnd/tls.cert
                newaddress p2wkh
                """
            ).output
        )
        treasuries[container.name] = new_address.address

    ### Open channels ###
    outputs: List[str] = []
    for container in track(paddles, "Open channels:".ljust(42)):
        if container.name == "aesir-ping":
            try:
                open_channel: OpenChannel = TypeAdapter(OpenChannel).validate_json(
                    container.exec_run(
                        """
                        lncli
                            --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                            --rpcserver=localhost:10001
                            --tlscertpath=/home/lnd/.lnd/tls.cert
                        openchannel %d
                            --node_key %s
                            --connect aesir-pong:9735
                        """
                        % (channel_size, nodekeys.get("aesir-pong", ""))
                    ).output
                )
                outputs.append(
                    f"<Channel 'aesir-ping --> aesir-pong' : txid='{ open_channel.funding_txid }'>"
                )
                bitcoind.exec_run(
                    """
                    bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir generatetoaddress %d %s
                    """
                    % (6, treasuries.get("aesir-ping", ""))
                )
            except ValidationError:
                outputs.append("[dim yellow1]Unable to open 'aesir-pong --> aesir-ping' channel.")
        elif container.name == "aesir-pong":
            try:
                open_channel: OpenChannel = TypeAdapter(OpenChannel).validate_json(  # type: ignore[no-redef]
                    container.exec_run(
                        """
                        lncli
                            --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
                            --rpcserver=localhost:10001
                            --tlscertpath=/home/lnd/.lnd/tls.cert
                        openchannel %d
                            --node_key %s
                            --connect aesir-ping:9735
                        """
                        % (channel_size, nodekeys.get("aesir-ping", ""))
                    ).output
                )
                outputs.append(
                    f"<Channel 'aesir-pong --> aesir-ping' : txid='{ open_channel.funding_txid }'>"
                )
                bitcoind.exec_run(
                    """
                    bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir generatetoaddress %d %s
                    """
                    % (6, treasuries.get("aesir-pong", ""))
                )
            except ValidationError:
                outputs.append("[dim yellow1]Unable to open 'aesir-pong --> aesir-ping' channel.")
    list(map(rich_print, outputs))


__all__ = ["ping_pong"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2023 All rights reserved.
# FILENAME:    ~~/src/commands/flush.py
# VERSION: 	   0.2.9
# CREATED: 	   2023-12-01 06:24
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import List, Set

### Third-party packages ###
from click import command
from docker import DockerClient, from_env
from docker.errors import DockerException
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from src.configs import DEPRECATED


@command
def flush() -> None:
    """Remove images deprecated by workspace."""
    client: DockerClient
    try:
        client = from_env()
        if not client.ping():
            raise DockerException
    except DockerException:
        rich_print("[red bold]Unable to connect to docker daemon.")
        return
    outputs: List[str] = []
    docker_images: Set[str] = {image.tags[0] for image in client.images.list()}
    for registry_id in track(DEPRECATED, "Remove deprecated images:".ljust(42)):
        if registry_id in docker_images:
            client.images.remove(registry_id)
            outputs.append(f"<Image: '{ registry_id }'> removed.")
    list(map(rich_print, outputs))


__all__ = ["flush"]
